#include <config.h>
#include <common.h>
#ifdef CONFIG_MX6DL
#include <asm/io.h>
#include <asm/arch/mx6.h>
#include <asm/arch/mx6_pins.h>
#include <asm/arch/mx6dl_pins.h>
#include <asm/arch/iomux-v3.h>
#include "ports.h"

static int  g_iTCK = 0; /* For xapp058_example .exe */
static int  g_iTMS = 0; /* For xapp058_example .exe */
static int  g_iTDI = 0; /* For xapp058_example .exe */

#define TMS_BIT_MASK    (1 << 7)
#define TCK_BIT_MASK    (1 << 8)
#define TDO_BIT_MASK    (1 << 9)
#define TDI_BIT_MASK    (1 << 5)

void setPort(short p,short val)
{
    int reg;
    switch (p) {
    case TMS:
        reg = readl(GPIO1_BASE_ADDR + GPIO_DR);
        if (val)
            reg |= TMS_BIT_MASK;
        else
            reg &= ~TMS_BIT_MASK;
        writel(reg, GPIO1_BASE_ADDR + GPIO_DR);
        g_iTMS = val;
        break;
    case TDI:
        reg = readl(GPIO4_BASE_ADDR + GPIO_DR);
        if (val) 
            reg |= TDI_BIT_MASK;
        else
            reg &= ~TDI_BIT_MASK;
        writel(reg, GPIO4_BASE_ADDR + GPIO_DR);
        g_iTDI = val;
        break;
    case TCK:
        reg = readl(GPIO1_BASE_ADDR + GPIO_DR);
        if (val) 
            reg |= TCK_BIT_MASK;
        else
            reg &= ~TCK_BIT_MASK;
        writel(reg, GPIO1_BASE_ADDR + GPIO_DR);
        g_iTCK = val;
        break;
    default:
        printf("setPort: Invalid port(%d)\n", p);
        break;
    }
}

void pulseClock()
{
    setPort(TCK,0);  /* set the TCK port to low  */
    setPort(TCK,1);  /* set the TCK port to high */
}

static unsigned char *gBuffer;
static unsigned int gIndex;

void setDataBuffer(unsigned char *buff)
{
    gBuffer = buff;
    gIndex = 0;
}

void readByte(unsigned char *data)
{
    *data   = gBuffer[gIndex];
    gIndex++;
}

/* readTDOBit:  Implement to return the current value of the JTAG TDO signal.*/
/* read the TDO bit from port */
unsigned char readTDOBit()
{
    int reg;
    reg = readl(GPIO1_BASE_ADDR + GPIO_PSR);
    return !!(reg & TDO_BIT_MASK);
}

void waitTime(long microsec)
{
    static long tckCyclesPerMicrosec    = 1; /* must be at least 1 */
    long        tckCycles   = microsec * tckCyclesPerMicrosec;
    long        i;

    for ( i = 0; i < tckCycles; ++i )
    {
        pulseClock();
    }
}

#endif
